@@ -1,3 +1,26 @@
+#  #####################################################################################
+#  R package stochvol by
+#     Gregor Kastner Copyright (C) 2013-2020
+#     Darjus Hosszejni Copyright (C) 2019-2020
+#  
+#  This file is part of the R package stochvol: Efficient Bayesian
+#  Inference for Stochastic Volatility Models.
+#  
+#  The R package stochvol is free software: you can redistribute it
+#  and/or modify it under the terms of the GNU General Public License
+#  as published by the Free Software Foundation, either version 2 or
+#  any later version of the License.
+#  
+#  The R package stochvol is distributed in the hope that it will be
+#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+#  General Public License for more details.
+#  
+#  You should have received a copy of the GNU General Public License
+#  along with the R package stochvol. If that is not the case, please
+#  refer to <http://www.gnu.org/licenses/>.
+#  #####################################################################################
+
 #' Graphical Summary of the Posterior Predictive Distribution
 #' 
 #' \code{plot.svpredict} and \code{plot.svlpredict} generate some plots
@@ -14,7 +37,6 @@
 #' used within \code{\link{plot.svdraws}} for a possibly more useful
 #' visualization. See the examples in \code{\link{predict.svdraws}} and
 #' those below for use cases.
-#' @author Gregor Kastner \email{gregor.kastner@@wu.ac.at}
 #' @family plotting
 #' @keywords hplot
 #' @examples
@@ -34,35 +56,38 @@
 #' ## Plot the latent volatilities and some forecasts
 #' plot(draws, forecast = pred)
 #' 
+#' @family plotting
+#' @keywords hplot
 #' @export
 plot.svpredict <- function(x, quantiles = c(.05, .25, .5, .75, .95), ...) {
+  nvar <- coda::nvar(x$h[[1]])
   oldpar <- par(mfrow = c(2, 1), mgp = c(1.8, .8, 0), mar = c(3, 3, 3, 1))
-  if (ncol(x$h) == 1L) {
-    boxplot(apply(exp(x$h / 2), 2, quantile, quantiles), names = "1 period ahead",
-	    show.names = TRUE, ...)
+  if (nvar == 1L) {
+    boxplot(simplify2array(x$vol, higher = FALSE),  # previous solution only worked with length(quantiles) == 5
+            names = paste("Next period / chain", seq_along(x$vol)),
+            show.names = TRUE, ...)
+    title("Predicted volatility")
   } else {
-    ts.plot(t(apply(exp(x$h / 2), 2, quantile, quantiles)), xlab = "Periods ahead", ...)
+    ts.plot(matrix(aperm(simplify2array(lapply(x$vol, apply, 2, quantile, quantiles)), c(2, 1, 3)), nrow = nvar),
+            xlab = "Periods ahead", ...)
+    title(paste0("Predicted volatility (", paste0(100*quantiles, collapse = '% / '),
+                 "% quantiles)"))
   }
-  title(paste0("Predicted volatility (", paste0(100*quantiles, collapse = '% / '),
-	       "% quantiles)"))
-  if (ncol(x$h) == 1L) {
-    boxplot(apply(x$y, 2, quantile, quantiles), names = "1 period ahead",
-	    show.names = TRUE, ...)
+  if (nvar == 1L) {
+    boxplot(simplify2array(x$y, higher = FALSE),
+            names = paste("Next period / chain", seq_along(x$y)),
+            show.names = TRUE, ...)
+    title("Predicted volatility")
   } else {
-    ts.plot(t(apply(x$y, 2, quantile, quantiles)), xlab = "Periods ahead", ...)
+    ts.plot(matrix(aperm(simplify2array(lapply(x$y, apply, 2, quantile, quantiles)), c(2, 1, 3)), nrow = nvar),
+            xlab = "Periods ahead", ...)
+    title(paste0("Predicted data (", paste0(100*quantiles, collapse = '% / '),
+                 "% quantiles)"))
   }
-  title(paste0("Predicted data (", paste0(100*quantiles, collapse = '% / '),
-	       "% quantiles)"))
   par(oldpar)
   invisible(x)
 }
 
-#' @rdname plot.svpredict
-#' @export
-plot.svlpredict <- function(x, quantiles = c(.05, .25, .5, .75, .95), ...) {
-  plot.svpredict(x, quantiles, ...)
-}
-
 #' Probability Density Function Plot for the Parameter Posteriors
 #' 
 #' Displays a plot of the density estimate for the posterior distribution of
@@ -73,7 +98,7 @@ plot.svlpredict <- function(x, quantiles = c(.05, .25, .5, .75, .95), ...) {
 #' \code{coda} package, with some modifications for parameters that have
 #' (half-)bounded support.
 #' 
-#' @param x \code{svdraws} or \code{svldraws} object.
+#' @param x \code{svdraws} object.
 #' @param showobs logical value, indicating whether the observations should be
 #' displayed along the x-axis. If many draws have been obtained, the default
 #' (\code{TRUE}) can render the plotting to be quite slow, and you might want
@@ -97,7 +122,6 @@ plot.svlpredict <- function(x, quantiles = c(.05, .25, .5, .75, .95), ...) {
 #' @return Called for its side effects. Returns argument \code{x} invisibly.
 #' @note You can call this function directly, but it is more commonly called by
 #' the \code{\link{plot.svdraws}} method.
-#' @author Gregor Kastner \email{gregor.kastner@@wu.ac.at}
 #' @family plotting
 #' @keywords hplot
 #' @export
@@ -105,30 +129,28 @@ paradensplot <- function(x, showobs = TRUE, showprior = TRUE, showxlab = TRUE,
                          mar = c(1.9, 1.9, 1.9, .5), mgp = c(2, .6, 0), simobj = NULL, ...) {
   if (!inherits(x, "svdraws")) stop("This function expects an 'svdraws' object.")
   if (!is.logical(showobs)) stop("If provided, argument 'showobs' must be TRUE or FALSE.")
-  if (!is.null(simobj)) {
-    if (!inherits(simobj, "svsim")) stop("If provided, simobj must be an 'svsim' object.")
-    sim <- TRUE
-  } else sim <- FALSE
-  oldpar <- par(mar=mar)
-  paranames <- c(mu=quote(mu), phi=quote(phi), sigma=quote(sigma), rho=quote(rho), nu=quote(nu))
-  cutat1 <- c(mu=FALSE, phi=TRUE, sigma=FALSE, rho=TRUE, nu=FALSE)
-  for (i in 1:ncol(x$para)) {
-    parastring <- colnames(x$para)[i]
-    mydensplot(x$para[,parastring], show.obs=showobs, main=paste("Density of", paranames[parastring]),
-               cutat1=cutat1[parastring], showxlab=showxlab, mgp = mgp, ...)
+  sim <- !is.null(simobj)
+  if (sim && !inherits(simobj, "svsim")) {
+    stop("If provided, simobj must be an 'svsim' object.")
+  }
+  oldpar <- par(mar = mar)
+  para_names <- c(mu = quote(mu), phi = quote(phi), sigma = quote(sigma), rho = quote(rho), nu = quote(nu))
+  para_prior_names <- c(mu = "mu", phi = "phi", sigma = "sigma2", rho = "rho", nu = "nu")
+  params <- sampled_parameters(x)
+  for (para_name in params) {
+    prior <- x$priors[[para_prior_names[para_name] ]]
+    cutat <- x$para_inv_transform[[para_name]](range(prior))
+    mydensplot(x$para[,para_name], show.obs=showobs, main=paste("Density of", para_names[para_name]),
+               cutat=cutat,
+               showxlab=showxlab, mgp = mgp, ...)
     if (isTRUE(showprior)) {
-      paras <- x$priors[[parastring]]
-      vals <- seq(from=par('usr')[1], to=par('usr')[2], len=1000)
-      if (parastring == "mu") lines(vals, dnorm(vals, paras[1], paras[2]), col=8, lty=2)
-      else if (parastring == "phi") lines(vals, .5*dbeta((vals+1)/2, paras[1], paras[2]), col=8, lty=2)
-      else if (parastring == "sigma") {
-        if (x$priors$gammaprior) lines(vals, 2*dnorm(vals, 0, sqrt(paras[1])), col=8, lty=2)
-        else lines(vals, 2*vals^(-3)*dgamma(1/vals^2, 3, 3), col=8, lty=2)
-      }
-      else if (parastring == "rho") lines(vals, .5*dbeta((vals+1)/2, paras[1], paras[2]), col=8, lty=2)
-      else if (parastring == "nu") lines(vals, dunif(vals, paras[1], paras[2]), col = 8, lty = 2)
-      if (sim && parastring %in% names(simobj$para)) {
-        points(simobj$para[[parastring]], 0, col = 3, cex = 2, pch = 16)
+      eps <- diff(par('usr'))[1] / 999
+      vals <- seq(from=max(c(par('usr')[1], cutat[1]+eps)), to=min(c(par('usr')[2], cutat[2]-eps)), len=1000)
+      lines(vals,
+            x$para_transform_det[[para_name]](vals) * density(prior)(x$para_transform[[para_name]](vals)),
+            col = 8, lty = 2)
+      if (sim) {
+        points(simobj$para[[para_name]], 0, col = 3, cex = 2, pch = 16)
       }
     }
   }
@@ -145,7 +167,7 @@ paradensplot <- function(x, showobs = TRUE, showprior = TRUE, showxlab = TRUE,
 #' \code{paratraceplot} is modeled after \code{\link[coda]{traceplot}} in the
 #' \code{coda} package, with very minor modifications.
 #' 
-#' @param x \code{svdraws} or \code{svldraws} object.
+#' @param x \code{svdraws} object.
 #' @param mar numerical vector of length 4, indicating the plot margins. See
 #' \code{\link[graphics]{par}} for details. The default value is \code{c(1.9,
 #' 1.9, 1.9, 0.5)}, which is slightly smaller than the R-defaults.
@@ -160,7 +182,6 @@ paradensplot <- function(x, showobs = TRUE, showprior = TRUE, showxlab = TRUE,
 #' @return Called for its side effects. Returns argument \code{x} invisibly.
 #' @note You can call this function directly, but it is more commonly called by
 #' the \code{\link{plot.svdraws}} method.
-#' @author Gregor Kastner \email{gregor.kastner@@wu.ac.at}
 #' @family plotting
 #' @keywords hplot
 #' @export
@@ -172,12 +193,12 @@ paratraceplot.svdraws <- function(x, mar = c(1.9, 1.9, 1.9, .5), mgp = c(2, .6,
   } else sim <- FALSE
   oldpar <- par(mar=mar)
   paranames <- c(mu=quote(mu), phi=quote(phi), sigma=quote(sigma), nu=quote(nu), rho=quote(rho))
-  for (i in 1:ncol(x$para)) {
-    parastring <- colnames(x$para)[i]
-    mytraceplot(x$para[,parastring], xlab="", mgp = mgp,
-                main=paste("Trace of ", paranames[parastring], " (thin = ", x$thinning$para,")", sep=''), ...)
-    if (sim && parastring %in% names(simobj$para)) {
-      abline(h = simobj$para[[parastring]], col = 3, lty = 2)
+  params <- sampled_parameters(x)
+  for (para_name in params) {
+    mytraceplot(para(x, "all")[,para_name], xlab="", mgp = mgp,
+                main=paste0("Trace of ", paranames[para_name], " (thin = ", x$thinning$para,")"), ...)
+    if (sim && para_name %in% names(simobj$para)) {
+      abline(h = simobj$para[[para_name]], col = 3, lty = 2)
     }
   }
   par(oldpar)
@@ -190,7 +211,7 @@ paratraceplot.svdraws <- function(x, mar = c(1.9, 1.9, 1.9, .5), mgp = c(2, .6,
 #' time as well as predictive distributions of future volatilities.
 #' 
 #' 
-#' @param x \code{svdraws} or \code{svldraws} object.
+#' @param x \code{svdraws} object.
 #' @param forecast nonnegative integer or object of class \code{svpredict}, as
 #' returned by \code{\link{predict.svdraws}}. If an integer greater than 0 is
 #' provided, \code{\link{predict.svdraws}} is invoked to obtain the
@@ -201,9 +222,6 @@ paratraceplot.svdraws <- function(x, mar = c(1.9, 1.9, 1.9, .5), mgp = c(2, .6,
 #' @param show0 logical value, indicating whether the initial volatility
 #' \code{exp(h_0/2)} should be displayed. The default value is \code{FALSE}.
 #' Only available for inputs \code{x} of class \code{svdraws}.
-#' @param col vector of color values (see \code{\link[graphics]{par}}) used for
-#' plotting the quantiles. The default value \code{NULL} results in gray lines
-#' for all quantiles expect the median, which is displayed in black.
 #' @param forecastlty vector of line type values (see
 #' \code{\link[graphics]{par}}) used for plotting quantiles of predictive
 #' distributions. The default value \code{NULL} results in dashed lines.
@@ -224,7 +242,7 @@ paratraceplot.svdraws <- function(x, mar = c(1.9, 1.9, 1.9, .5), mgp = c(2, .6,
 #' @param newdata corresponds to parameter \code{newdata} in \code{\link{predict.svdraws}}.
 #' \emph{Only if \code{forecast} is a positive integer and \code{\link{predict.svdraws}}
 #' needs a \code{newdata} object.} Corresponds to input
-#' parameter \code{designmatrix} in \code{\link{svsample}} and \code{\link{svlsample}}.
+#' parameter \code{designmatrix} in \code{\link{svsample}}.
 #' A matrix of regressors with number of rows equal to parameter \code{forecast}.
 #' @return Called for its side effects. Returns argument \code{x} invisibly.
 #' @note In case you want different quantiles to be plotted, use
@@ -255,34 +273,37 @@ paratraceplot.svdraws <- function(x, mar = c(1.9, 1.9, 1.9, .5), mgp = c(2, .6,
 #' 
 #' @export
 volplot <- function(x, forecast = 0, dates = NULL, show0 = FALSE,
-                    col = NULL, forecastlty = NULL, tcl = -.4,
+                    forecastlty = NULL, tcl = -.4,
                     mar = c(1.9, 1.9, 1.9, .5), mgp = c(2, .6, 0), simobj = NULL,
                     newdata = NULL, ...) {
-  if (!inherits(x, "svdraws")) stop("This function expects an 'svdraws' or an 'svldraws' object.")
+  if (!inherits(x, "svdraws")) stop("This function expects an 'svdraws' object.")
   if (x$thinning$time != "all") stop("This function requires that all volatilities have been stored during sampling.")
   if (!is.null(simobj)) {
     if (!inherits(simobj, "svsim")) stop("If provided, simobj must be an 'svsim' object.")
     sim <- TRUE
   } else sim <- FALSE
   oldpar <- par(mar = mar)
-  where <- grep("%", dimnames(x$summary$latent)[[2]])
-  volquants <- t(100*exp(x$summary$latent[,where,drop=FALSE]/2))  # monotone transformation!
-  nvolquants <- dim(volquants)[1]
-  timelen <- dim(volquants)[2]
+  to_plot <- x$summary$sd
+  where <- grep("%", colnames(to_plot))
+  volquants <- t(100*to_plot[,where,drop=FALSE])
+  nvolquants <- NROW(volquants)
+  timelen <- NCOL(volquants)
   if (is.null(nvolquants) || all(is.na(volquants))) stop("No quantiles to plot.")
-  if (is.null(col)) {
-    cols <- rep(8, nvolquants)
-    cols[dimnames(volquants)[[1]] == "50%"] <- 1
-  } else cols <- col
+  alphas <- 1 - 1.2 * abs(0.5 - as.numeric(gsub("%", "", colnames(to_plot)[grep("%", colnames(to_plot))]))/100)
+  # set alpha for colors
+  cols <- cbind(1, alphas)
+  cols <- apply(cols, 1, function (x) {
+                  rgb_cols <- as.vector(col2rgb(x[1]))/255
+                  rgb(rgb_cols[1], rgb_cols[2], rgb_cols[3], alpha=x[2])})
   if (is.null(forecastlty)) forecastlty <- 2
 
   if (inherits(forecast, "svpredict") || (is.numeric(forecast) && length(forecast) == 1 && all(forecast != 0))) { # also draw future values
-    lasth <- as.integer(gsub("h_", "", dimnames(x$latent)[[2]][dim(x$latent)[2]]))
+    lasth <- as.integer(gsub("h_", "", tail(coda::varnames(x$latent[[1]]), 1)))
     if (length(x$y) > lasth) {  # should never happen
       stop("The last log variance, h_n, has not been stored during sampling. Aborting.")
     }
 
-    if(is.numeric(forecast) && length(forecast) == 1 && all(forecast >= 1)) {
+    if(is.numeric(forecast) && length(forecast) == 1 && isTRUE(forecast >= 1)) {
       forecast <- predict(x, forecast, newdata)
     }
     if(!inherits(forecast, "svpredict")) stop("Argument 'forecast' must be a single nonnegative integer, or of class type 'svpredict' or 'svlpredict'.")
@@ -291,12 +312,12 @@ volplot <- function(x, forecast = 0, dates = NULL, show0 = FALSE,
       show0 <- FALSE
     }
 
-    volpred <- forecast$h
+    volpred <- join_mcmclist(forecast$vol)
     futlen <- NCOL(volpred)
 
     xs <- matrix(rep(seq(timelen, timelen + futlen, len=futlen+1), nvolquants), nrow=futlen+1)
-    quants <- as.numeric(gsub("%", "", dimnames(volquants)[[1]]))/100
-    ys <- rbind(volquants[,timelen], t(matrix(apply(100*exp(volpred/2), 2, quantile, quants), nrow=nvolquants)))
+    quants <- as.numeric(gsub("%", "", rownames(volquants)))/100
+    ys <- rbind(volquants[,timelen], t(matrix(apply(100*volpred, 2, quantile, quants), nrow=nvolquants)))
 
     if (futlen > .01*timelen) {  # increase xlim to give space for forecast
       xlim <- c(0, timelen + futlen)
@@ -305,23 +326,17 @@ volplot <- function(x, forecast = 0, dates = NULL, show0 = FALSE,
     }
   } else xlim <- NULL
 
-  if(exists("sd", x$summary)) {  # heavy-tailed innovation
-    mymain <- paste("Estimated scaling in percent (", paste(dimnames(volquants)[[1]], collapse=' / '),
-                    " posterior quantiles)", sep = '')
-  } else {
-    mymain <- paste("Estimated volatilities in percent (", paste(dimnames(volquants)[[1]], collapse=' / '),
-                    " posterior quantiles)", sep = '')
-  }
-
   ts.plot(t(volquants), gpars=list(xlim=xlim, col=cols, xlab='', xaxt='n', mgp=mgp, tcl=tcl,
-                                   main = mymain, ...))
+                                   main = paste("Estimated volatilities in percent (", paste(dimnames(volquants)[[1]], collapse=' / '),
+                                                " posterior quantiles)", sep = ''),
+                                   ...))
 
   if (sim) {
     lines(100*simobj$vol, col = 3)
   }
 
   if (inherits(forecast, "svpredict")) {
-    for (i in 1:nvolquants) lines(xs[,i], ys[,i], lty=forecastlty, col=cols[i])
+    for (i in seq_len(nvolquants)) lines(xs[,i], ys[,i], lty=forecastlty, col=cols[i])
   }
 
   ax <- axis(1, tick=FALSE, labels=FALSE)  # just automagic axis ticks, don't draw yet
@@ -330,47 +345,24 @@ volplot <- function(x, forecast = 0, dates = NULL, show0 = FALSE,
     xs <- matrix(rep(c(0, 1), nvolquants), nrow=2)
     where <- grep("%", names(x$summary$latent0))
     ys <- rbind(100*exp(x$summary$latent0[where]/2), volquants[,1])
-    for (i in 1:nvolquants) lines(xs[,i], ys[,i], lty=forecastlty, col=cols[i])
+    for (i in seq_len(nvolquants)) lines(xs[,i], ys[,i], lty=forecastlty, col=cols[i])
   }
 
   if (is.null(dates)) {
-    dates <- c(0L, as.integer(gsub("h_", "", dimnames(x$latent)[[2]])))
+    dates <- c(0L, as.integer(gsub("h_", "", coda::varnames(latent(x)))))
     if (max(ax) > length(dates)) {  # means we are probably forecasting and need extra axis labels
       dates <- c(dates, seq(length(dates), max(ax), by=dates[2]-dates[1]))
     }
   } else {
     if (inherits(dates, "Date")) dates <- as.character(dates)
-    if (length(dates) != ncol(x$latent)) {
-      stop("Length of argument 'dates' differs from ncol(x$latent).")
+    if (length(dates) != NCOL(latent(x, 1))) {
+      stop("Length of argument 'dates' differs from NCOL(latent(x, 1)).")
     }
     dates <- c('', dates)
     ax <- ax[ax != 0]  # avoid "zero" tick
   }
   axis(1, at=ax, labels=dates[ax+1], mgp=mgp, tcl=tcl)
 
-  if(exists("sd", x$summary)) {  # only for t-distributed residuals
-    where <- grep("%", dimnames(x$summary$latent)[[2]])
-    ts.plot(100*x$summary$sd[,where], gpars=list(xlim=xlim, col=cols, xlab='', xaxt='n', mgp=mgp, tcl=tcl,
-                                                 main = paste("Estimated volatilities in percent (",
-                                                              paste(dimnames(volquants)[[1]], collapse=' / '),
-                                                              " posterior quantiles)", sep=''), ...))
-
-    if (sim) {
-      standardizer <- sqrt(simobj$para$nu / (simobj$para$nu - 2))
-      lines(100*simobj$vol*standardizer, col = 3)
-    }
-
-    if (inherits(forecast, "svpredict")) {
-      standardizer <- sqrt(x$para[,"nu"] / (x$para[,"nu"] - 2))
-      where <- grep("%", dimnames(x$summary$latent)[[2]])
-      ys <- rbind(100*x$summary$sd[timelen,where,drop=FALSE],
-                  t(matrix(apply(100*exp(volpred/2)*standardizer, 2, quantile, quants), nrow=nvolquants)))
-
-      for (i in 1:nvolquants) lines(xs[,i], ys[,i], lty=forecastlty, col=cols[i])
-    }
-    axis(1, at=ax, labels=dates[ax+1], mgp=mgp, tcl=tcl)
-  }
-
   par(oldpar)
   invisible(x)
 }
@@ -385,7 +377,7 @@ volplot <- function(x, forecast = 0, dates = NULL, show0 = FALSE,
 #' These functions set up the page layout and call \code{\link{volplot}},
 #' \code{\link{paratraceplot}} and \code{\link{paradensplot}}.
 #' 
-#' @param x \code{svdraws} or \code{svldraws} object.
+#' @param x \code{svdraws} object.
 #' @param forecast nonnegative integer or object of class \code{svpredict}, as
 #' returned by \code{\link{predict.svdraws}}. If an integer greater than 0 is
 #' provided, \code{\link{predict.svdraws}} is invoked to obtain the
@@ -402,9 +394,6 @@ volplot <- function(x, forecast = 0, dates = NULL, show0 = FALSE,
 #' to try setting \code{showobs} to \code{FALSE}.
 #' @param showprior logical value, indicating whether the prior distribution
 #' should be displayed. The default value is \code{TRUE}.
-#' @param col vector of color values (see \code{\link[graphics]{par}}) used for
-#' plotting the quantiles. The default value \code{NULL} results in gray lines
-#' for all quantiles expect the median, which is displayed in black.
 #' @param forecastlty vector of line type values (see
 #' \code{\link[graphics]{par}}) used for plotting quantiles of predictive
 #' distributions. The default value \code{NULL} results in dashed lines.
@@ -423,7 +412,7 @@ volplot <- function(x, forecast = 0, dates = NULL, show0 = FALSE,
 #' @param newdata corresponds to parameter \code{newdata} in \code{\link{predict.svdraws}}.
 #' \emph{Only if \code{forecast} is a positive integer and \code{\link{predict.svdraws}}
 #' needs a \code{newdata} object.} Corresponds to input
-#' parameter \code{designmatrix} in \code{\link{svsample}} and \code{\link{svlsample}}.
+#' parameter \code{designmatrix} in \code{\link{svsample}}.
 #' A matrix of regressors with number of rows equal to parameter \code{forecast}.
 #' @param \dots further arguments are passed on to the invoked plotting
 #' functions.
@@ -451,52 +440,52 @@ volplot <- function(x, forecast = 0, dates = NULL, show0 = FALSE,
 #' newquants <- c(0.01, 0.05, 0.25, 0.5, 0.75, 0.95, 0.99)
 #' draws <- updatesummary(draws, quantiles = newquants)
 #' 
-#' plot(draws, forecast = 20, showobs = FALSE, col = seq(along = newquants),
+#' plot(draws, forecast = 20, showobs = FALSE,
 #'      forecastlty = 3, showprior = FALSE)
 #' 
 #' @export
 plot.svdraws <- function(x, forecast = NULL, dates = NULL,
-			 show0 = FALSE, showobs = TRUE, showprior = TRUE, col = NULL,
+			 show0 = FALSE, showobs = TRUE, showprior = TRUE,
 			 forecastlty = NULL, tcl = -0.4,
 			 mar = c(1.9, 1.9, 1.7, .5), mgp = c(2, .6, 0),
 			 simobj = NULL, newdata = NULL, ...) {
-  oldpar <- par(mfrow=c(1,1))
-  if (x$thinning$time == "all") {
-    if (ncol(x$para) == 4) {
-      layout(matrix(c(1, 1, 1, 1, 2, 2, 2, 2, 3, 4, 5, 6, 7, 8, 9, 10), 4, byrow = TRUE))
-    } else {
-      layout(matrix(c(1, 1, 1, 2, 3, 4, 5, 6, 7), 3, byrow = TRUE))
-    }
-    volplot(x, dates = dates, show0 = show0, forecast = forecast,
-            forecastlty = forecastlty, col = col, tcl = tcl, mar = mar,
-            mgp = mgp, simobj = simobj, newdata = newdata, ...)
-  } else {  # don't plot volatility
-    layout(matrix(c(1, 2, 3, 4, 5, 6), 2, byrow = TRUE))
+  # Helper values
+  params <- sampled_parameters(x)
+  heavy_tailed_sv <- "nu" %in% params
+  plot_volatility_series <- thinning(x)$time == "all"
+  npara <- length(params)
+  if (is.null(simobj) && !is.null(x$simobj)) {
+    message("Simulation object extracted from input")
+    simobj <- x$simobj
   }
-  paratraceplot(x, mar = mar, mgp = mgp, simobj = simobj, ...)
-  paradensplot(x, showobs = showobs, showprior = showprior,
-               showxlab = FALSE, mar = mar, mgp = mgp, simobj = simobj, ...)
-  par(oldpar)
-  invisible(x)
-}
+  
+  # Set layout
+  index <- 1L
+  chart_indices <- c()
+  if (plot_volatility_series) {  # volatility chart(s)
+    chart_indices <- c(chart_indices, rep_len(index, length.out = npara))
+    index <- index + 1L
+  }
+  if (!x$resampled) {
+    chart_indices <- c(chart_indices, index - 1L + seq_len(npara))  # parameter trace plots
+    index <- index + npara
+  }
+  chart_indices <- c(chart_indices, index - 1L + seq_len(npara))  # parameter density plots
+  index <- index + npara
+  oldpar <- par(mfrow=c(1,1))  # 'layout' and 'par' do not play well together
+  layout(matrix(chart_indices, ncol = npara, byrow = TRUE))
 
-#' @rdname plot.svdraws
-#' @export
-plot.svldraws <- function (x, forecast = NULL, dates = NULL,
-			 show0 = FALSE, showobs = TRUE, showprior = TRUE, col = NULL,
-			 forecastlty = NULL, tcl = -0.4,
-			 mar = c(1.9, 1.9, 1.7, .5), mgp = c(2, .6, 0),
-			 simobj = NULL, newdata = NULL, ...) {
-  oldpar <- par(mfrow=c(1,1))
-  if (x$thinning$time == "all") {
-    layout(matrix(c(1, 1, 1, 1, 2, 3, 4, 5, 6, 7, 8, 9), ncol=4, byrow = TRUE))
+  # Plotting
+  if (plot_volatility_series) {
     volplot(x, dates = dates, show0 = show0, forecast = forecast,
-            forecastlty = forecastlty, col = col, tcl = tcl, mar = mar,
+            forecastlty = forecastlty, tcl = tcl, mar = mar,
             mgp = mgp, simobj = simobj, newdata = newdata, ...)
-  } else {  # don't plot volatility
-    layout(matrix(c(1, 2, 3, 4, 5, 6, 7, 8), 2, byrow = TRUE))
   }
-  paratraceplot(x, mar = mar, mgp = mgp, simobj = simobj, ...)
+  if (!x$resampled) {
+    paratraceplot(x, mar = mar, mgp = mgp, simobj = simobj, ...)
+  } else {
+    message("Traceplots are not shown after re-sampling")
+  }
   paradensplot(x, showobs = showobs, showprior = showprior,
                showxlab = FALSE, mar = mar, mgp = mgp, simobj = simobj, ...)
   par(oldpar)
@@ -504,105 +493,80 @@ plot.svldraws <- function (x, forecast = NULL, dates = NULL,
 }
 
 # modified density plot (from coda package)
-mydensplot <- function(x, show.obs = TRUE, bwf, main = "", ylim, cutat1=FALSE, showxlab=TRUE, mgp = c(2,.6,0), tcl=-.4, ...) {
-  for (i in 1:nvar(x)) {
-    x.i <- as.matrix(x)[, i, drop = TRUE]
-    if (missing(bwf)) 
+# x is an mcmc.list with a single variable
+mydensplot <- function(x, show.obs = TRUE, bwf, main = "", ylim, cutat=c(-Inf, Inf), showxlab=TRUE, mgp = c(2,.6,0), tcl=-.4, ...) {
+  varname <- coda::varnames(x[[1]])
+  draws <- coda::niter(x[[1]])
+  for (i in seq_along(x)) {
+    x_i <- as.vector(x[[i]])
+    range_x_i <- range(x_i)
+    if (range_x_i[1] < cutat[1] || range_x_i[2] > cutat[2]) {
+      stop("Argument 'cutat' does not include range of variable.")
+    }
+    if (missing(bwf)) {
       bwf <- function(xx) {
         xx <- xx[!is.na(as.vector(xx))]
         return(1.06 * min(sd(xx), IQR(xx)/1.34) * length(xx)^-0.2)
       }
-    bw <- bwf(x.i)
-    width <- 4 * bw
-    if (max(abs(x.i - floor(x.i))) == 0 || bw == 0) {
-      hist(x.i, prob = TRUE, main = main, ...)
-    } else {
-      density.scale <- "open"
-      if (isTRUE(cutat1)) {
-        if (1-max(x.i) < 2*bw) {
-          density.scale <- "cutat1"
-          x.i <- c(x.i, 2 - x.i)
-          if (1+min(x.i) < 2*bw) {
-            density.scale <- "cutatboth"
-            x.i <- c(x.i, -2 - x.i, 2 - x.i)
-          }
-        } else if (1+min(x.i) < 2*bw) {
-          density.scale <- "cutat-1"
-          x.i <- c(x.i, -2 - x.i)
-        }
-      } else if (max(x.i) <= 1 && 1 - max(x.i) < 2 * bw) {
-        if (min(x.i) >= 0 && min(x.i) < 2 * bw) {
-          density.scale <- "proportion"
-          x.i <- c(x.i, -x.i, 2 - x.i)
-        }
-      } else if (min(x.i) >= 0 && min(x.i) < 2 * bw) {
-        density.scale <- "positive"
-        x.i <- c(x.i, -x.i)
-      }
-      dens <- density(x.i, width = width)
-
-      if (density.scale == "proportion") {
-        dens$y <- 3 * dens$y[dens$x >= 0 & dens$x <= 1]
-        dens$x <- dens$x[dens$x >= 0 & dens$x <= 1]
-      }
-      else if (density.scale == "positive") {
-        dens$y <- 2 * dens$y[dens$x >= 0]
-        dens$x <- dens$x[dens$x >= 0]
-      }
-      else if (density.scale == "cutat1") {
-        dens$y <- 2 * dens$y[dens$x <= 1]
-        dens$x <- dens$x[dens$x <= 1]
-      }
-      else if (density.scale == "cutat-1") {
-        dens$y <- 2 * dens$y[dens$x >= -1]
-        dens$x <- dens$x[dens$x >= -1]
-      }
-      else if (density.scale == "cutatboth") {
-        dens$y <- 3 * dens$y[dens$x >= -1 & dens$x <= 1]
-        dens$x <- dens$x[dens$x >= -1 & dens$x <= 1]
-      }
-
-      if (missing(ylim)) 
-        ylim <- c(0, max(dens$y))
-
-      plot(dens, ylab = "", main = main, type = "l", 
-           ylim = ylim, xlab="", mgp = mgp, tcl = tcl, ...)
-      if(isTRUE(showxlab)) {
-        if (is.R()) {
-          mtext(paste("N =", niter(x), "  Bandwidth =",
-                      formatC(dens$bw)), side=1, line=2.7, cex=.7)
-        } else {
-          mtext(paste("N =", niter(x), "  Bandwidth =",
-                      formatC(bw)), side=1, line=2.7, cex=.7)
-        }
-      }
-      if (show.obs) 
-        lines(x.i[1:niter(x)], rep(max(dens$y)/100, niter(x)), 
-              type = "h")
     }
-    if (!is.null(varnames(x)) && is.null(list(...)$main)) 
-      title(paste("Density of", varnames(x)[i]))
+    bw <- bwf(x_i)
+    width <- 4 * bw
+    if (max(abs(x_i - floor(x_i))) == 0 || bw == 0) {
+      hist(x_i, prob = TRUE, main = main, col = i, ...)
+    } else {
+      density_scale <- "open"
+      cut_at_bottom <- isTRUE(is.finite(cutat[1]) && range_x_i[1]-cutat[1] < 2*bw)
+      cut_at_top <- isTRUE(is.finite(cutat[2]) && cutat[2]-range_x_i[2] < 2*bw)
+      x_i <- if (cut_at_bottom && cut_at_top) {
+        c(x_i, 2*cutat[1] - x_i, 2*cutat[2] - x_i)
+      } else if (cut_at_bottom) {
+        c(x_i, 2*cutat[1] - x_i)
+      } else if (cut_at_top) {
+        c(x_i, 2*cutat[2] - x_i)
+      } else {
+        x_i
+      }
+      upscale <- 1 + cut_at_bottom + cut_at_top
+      dens <- density(x_i, width = width)
+      index <- dens$x >= cutat[1] & dens$x <= cutat[2]
+      dens$x <- dens$x[index]
+      dens$y <- upscale * dens$y[index]
+
+      if (missing(ylim)) {
+        ylim <- c(0, max(dens$y))
+      }
+      if (i == 1) {
+        plot(dens, ylab = "", main = main, type = "l", 
+             ylim = ylim, xlab="", mgp = mgp, tcl = tcl, col = i, ...)
+      } else {
+        lines(dens$x, dens$y, type = "l", col = i)
+      }
+      if (show.obs) {
+        lines(x_i[seq_len(draws)], rep(max(dens$y)/100, draws),
+              type = "h", col = i)
+      }
+    }
+  }
+  if(isTRUE(showxlab)) {
+    if (is.R()) {
+      mtext(paste("N =", coda::niter(x[[1]]), "  Bandwidth =",
+                  formatC(dens$bw)), side=1, line=2.7, cex=.7)
+    } else {
+      mtext(paste("N =", coda::niter(x[[1]]), "  Bandwidth =",
+                  formatC(bw)), side=1, line=2.7, cex=.7)
+    }
   }
   invisible(x)
 }
 
 # modified traceplot (from coda)
-mytraceplot <- function (x, smooth = FALSE, col = 1:6, type = "l", ylab = "", xlab = "Iterations", mgp = c(2,.6,0), tcl = -.4, ...) {
-  x <- mcmc.list(x)
-  for (j in 1:nvar(x)) {
-    xp <- as.vector(time(x))
-    yp <- if (nvar(x) > 1) x[, j, drop = TRUE] else x
-    yp <- do.call("cbind", yp)
-    matplot(xp, yp, xlab = xlab, ylab = ylab, type = type, 
-            col = col, mgp = mgp, tcl = tcl, ...)
-    if (!is.null(varnames(x)) && is.null(list(...)$main)) 
-      title(paste("Trace of ", varnames(x)[j], " (thin = ", attr(x, "thinning")$thinpara,")", sep=''))
-    if (smooth) {
-      scol <- rep(col, length = nchain(x))
-      for (k in 1:nchain(x))
-        lines(lowess(xp, yp[, k]), col = scol[k])
-    }
-  }
+# x is an mcmc.list with a single variable
+mytraceplot <- function (x, type = "l", ylab = "", xlab = "Iterations", mgp = c(2,.6,0), tcl = -.4, ...) {
+  varname <- coda::varnames(x[[1]])
+  xp <- as.vector(time(x[[1]]))
+  yp <- simplify2array(x)
+  graphics::matplot(xp, yp, xlab = xlab, ylab = ylab, type = type, 
+                    col = seq_along(x), mgp = mgp, tcl = tcl, ...)
   invisible(x)
 }
 
@@ -641,10 +605,11 @@ plot.svresid <- function(x, origdata = NA,
   if (!terr) abline(h=qnorm(c(.025, .975)), lty=2)
   where <- seq(1, length(x), length=min(7, length(x)))
   axis(1, at = where, labels = gsub("r_", "", names(x)[where]))
-  qqplot(qt(ppoints(length(x)), df = nu), x,
+  qqplot(qt(ppoints(length(x)), df = nu), sqrt(nu / (nu-2))*x,
          main=paste(mains[2], "for", attr(x, "type"), "standardized residuals"),
          xlab = xlab, ylab = "Sample quantiles")
   qqline(x, probs = c(0.01, 0.99), distribution = function(x) qt(x, df = nu))
   par(oldpar)
   invisible(x)
 }
+
