@@ -32,7 +32,7 @@
 #include "sampling_latent_states.h"
 #include "utils.h"
 #include "utils_latent_states.h"
-#include "type_definitions.h"
+#include <type_definitions.h>
 
 using namespace Rcpp;
 
