@@ -33,7 +33,7 @@
 #include "utils.h"
 #include "utils_parameters.h"
 #include "utils_latent_states.h"
-#include "type_definitions.h"
+#include <type_definitions.h>
 #include "densities.h"
 #include <array>
 #include <utility>
